/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.Serializable;
import java.security.MessageDigest;
import pp4u_manage.ManageConfigure;

/**
 * キー情報
 * @author tondol
 */
public class InfoKey implements Serializable {

    private String filename,  digest;
    private long modified,  size;
    private InfoNode node;
    private boolean isLocalKey = true;

    /**
     * FileからKeyInfoを生成
     */
    public InfoKey(File file) {
        if (!file.isFile()) {
            return;
        }
        try {
            this.filename = file.getName();
            this.size = file.length();
            this.modified = file.lastModified();
            this.node = AppMain.getLocalhost();
            this.digest = getDigest(file);
        } catch (Exception ex) {
        }
    }

    /**
     * キーのフルパスを取得(ローカルのみ)
     */
    public String getUploadPath() {
        ManageConfigure manager = ManageConfigure.getInstance();
        return manager.getUpload() + File.separator + filename;
    }

    public String getDownloadPath() {
        ManageConfigure manager = ManageConfigure.getInstance();
        return manager.getDownload() + File.separator + filename;
    }

    /**
     * 検索ワードに一致するかどうか
     */
    public boolean matches(String word) {
        if (filename.indexOf(word) == -1) {
            return false;
        }
        return true;
    }

    /**
     * 検索ワード配列に一致するかどうか
     */
    public boolean matches(String[] words) {
        for (String word : words) {
            if (filename.indexOf(word) == -1) {
                if (word.charAt(0) != '-') {// 不一致かつ，NOT検索でない
                    return false;
                }
            } else {
                if (word.charAt(0) == '-') {// 一致かつ，NOT検索である
                    return false;
                }
            }
        }
        return true;
    }

    /**
     * InputStreamからDigestを計算
     */
    static private final String getDigest(File file) {
        FileInputStream in = null;
        MessageDigest sha = null;
        try {
            in = new FileInputStream(file);
            sha = MessageDigest.getInstance("SHA-1");
            byte[] buff = new byte[65536];
            int length = in.read(buff);
            while (length > 0) {
                sha.update(buff, 0, length);
                length = in.read(buff);
            }
        } catch (Exception ex) {
        } finally {
            try {
                if (in != null) {
                    in.close();
                }
            } catch (IOException iex) {
            }
        }
        return toHexString(sha.digest());
    }

    /**
     * byte配列をStringに変換
     */
    static private String toHexString(byte[] bytes) {
        StringBuffer buff = new StringBuffer();
        for (byte b : bytes) {
            buff.append(String.format("%1$02x", b));
        }
        return buff.toString();
    }

    /**
     * キーの種別を返す
     */
    public boolean isLocalKey() {
        return isLocalKey;
    }

    /**
     * キーの種別を設定
     */
    public void setLocalKey(boolean isLocalKey) {
        this.isLocalKey = isLocalKey;
    }

    public String getFilename() {
        return filename;
    }

    public long getSize() {
        return size;
    }

    public String getDigest() {
        return digest;
    }

    public long getModified() {
        return modified;
    }

    public InfoNode getNode() {
        return node;
    }

    public void setNode(InfoNode node) {
        this.node = node;
    }

    //////////////////////////////////////////////
    // インターフェイスの実装
    //////////////////////////////////////////////
    /**
     * ファイル名，ハッシュ，送信元でidentify
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == null || getClass() != obj.getClass()) {
            return false;
        }
        final InfoKey other = (InfoKey) obj;
        if (!this.filename.equals(other.filename)) {
            return false;
        }
        if (!this.digest.equals(other.digest)) {
            return false;
        }
        if (!this.node.equals(other.node)) {
            return false;
        }
        return true;
    }

    /**
     * ハッシュ値を計算
     */
    @Override
    public int hashCode() {
        int hash = 3;
        hash = 97 * hash + this.filename.hashCode();
        hash = 97 * hash + this.digest.hashCode();
        hash = 97 * hash + this.node.hashCode();
        return hash;
    }
}
