/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u;

import java.io.Serializable;
import pp4u_manage.ManageConfigure;
import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * 接続先の表現
 * @author tondol
 */
public class InfoNode implements Serializable {

    private String hostname;
    private int port;

    /**
     * ドメインとポート番号から初期化
     */
    public InfoNode(String hostname, int port) {
        this.hostname = hostname;
        this.port = port;
    }

    /**
     * ホストとポートとDDNS名を指定して初期化
     */
    public InfoNode(String hostname, int port, String dynamic) {
        if (dynamic.length() == 0) {
            this.hostname = hostname;
        } else {
            this.hostname = dynamic;
        }
        this.port = port;
    }

    /**
     * 「ドメイン:ポート番号」形式の文字列で初期化
     */
    public InfoNode(String node) {
        String[] array = node.split(":", 2);
        this.hostname = array[0];
        this.port = Integer.parseInt(array[1]);
    }

    /**
     * ローカルアドレスを取得
     */
    static public InfoNode getLocalhost() {
        InfoNode localhost = null;
        try {
            String address = InetAddress.getLocalHost().getHostAddress();
            String dynamic = ManageConfigure.getInstance().getDynamic();
            int port = ManageConfigure.getInstance().getPort();
            localhost = new InfoNode(address, port, dynamic);
        } catch (UnknownHostException uex) {
        }
        return localhost;
    }

    public String getHostname() {
        return hostname;
    }

    public int getPort() {
        return port;
    }

    public void setHostname(String hostname) {
        this.hostname = hostname;
    }

    public void setPort(int port) {
        this.port = port;
    }

    @Override
    public String toString() {
        return hostname + ":" + port;
    }

    //////////////////////////////////////////////
    // インターフェイスの実装
    //////////////////////////////////////////////
    /**
     * ホスト名とポートでidentify
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == null || getClass() != obj.getClass()) {
            return false;
        }
        final InfoNode other = (InfoNode) obj;
        if (!this.hostname.equals(other.hostname)) {
            return false;
        }
        if (this.port != other.port) {
            return false;
        }
        return true;
    }

    /**
     * ハッシュ値を計算
     */
    @Override
    public int hashCode() {
        int hash = 3;
        hash = 97 * hash + this.hostname.hashCode();
        hash = 97 * hash + this.port;
        return hash;
    }
}
