/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u;

import java.io.BufferedInputStream;
import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.net.SocketException;
import pp4u_packet.Packet;
import pp4u_packet.PacketGet;
import pp4u_packet.PacketPing;
import pp4u_packet.PacketPong;
import pp4u_packet.PacketPut;

/**
 * 送信タスク
 * @author tondol
 */
public class TaskUpload extends TaskAbstract implements Runnable {

    private Socket socket = null;
    private ObjectInputStream in = null;
    private ObjectOutputStream out = null;
    private InfoNode gaddr = null;
    private InfoNode paddr = null;
    private volatile Thread thread = null;
    private InfoKey key = null;
    private long current = 0;
    private long length = 0;

    public TaskUpload(Socket socket, ObjectInputStream in, PacketPing ping) throws Exception {
        try {
            // PONG送信
            this.socket = socket;
            this.in = in;
            this.out = new ObjectOutputStream(socket.getOutputStream());
            PacketPong pong = new PacketPong(PacketPong.OK);
            send(pong);
            
            // ホスト情報を取得
            String ghost = socket.getInetAddress().getHostAddress();
            int gport = pong.getAddress().getPort();
            this.paddr = pong.getAddress();
            this.gaddr = new InfoNode(ghost, gport);

            thread = new Thread(this);
            thread.start();
        } catch (Exception ex) {
            stop();
            throw ex;
        }
    }

    public void stop() {
        try {
            if (socket != null) {
                socket.close();
            }
        } catch (IOException iex) {
        } finally {
            socket = null;
            thread = null;
        }
    }

    public void run() {
        FileInputStream fst = null;
        BufferedInputStream ist = null;
        byte[] buffer = null;
        int blockLength = 0;

        try {
            while (socket.isConnected()) {
                // GET受信
                PacketGet get = (PacketGet) recv();
                // 初回GETのみ真面目に処理
                if (key == null) {
                    this.key = get.getKey();
                    this.length = key.getSize();
                    this.current = get.getIndex();
                    // ブロックの長さを取得
                    blockLength = get.getLength();
                    buffer = new byte[blockLength];
                    // ファイルを開く
                    File file = new File(key.getUploadPath());
                    fst = new FileInputStream(file);
                    ist = new BufferedInputStream(fst);
                    ist.skip(current);
                }
                // ブロックを読み込む
                int loaded = (int) Math.min(length - current, blockLength);
                ist.read(buffer, 0, loaded);
                // PUT送信
                PacketPut put = new PacketPut(key, current, buffer);
                send(put);
                current += loaded;
            }
        } catch (SocketException sex) {
        } catch (EOFException eex) {
            // 正常終了で発生
        } catch (Exception ex) {
            AppMain.debug("# 送信タスクエラー");
            AppMain.debug("@ " + ex);
        } finally {
            try {
                if (ist != null) {
                    ist.close();
                }
            } catch (IOException iex) {
            } finally {
                stop();
            }
        }
    }

    private void send(Packet packet) throws IOException {
        out.writeObject(packet);
        out.reset();
    }

    private Packet recv() throws IOException, ClassNotFoundException {
        return (Packet) in.readObject();
    }

    //////////////////////////////////////////////
    // 取得・設定メソッド
    //////////////////////////////////////////////
    @Override
    public boolean isAlive() {
        if (thread == null) {
            return false;
        } else {
            return true;
        }
    }

    @Override
    public String getType() {
        return "送信タスク";
    }

    @Override
    public String getFilename() {
        if (key == null) {
            return null;
        } else {
            return key.getFilename();
        }
    }

    @Override
    public InfoNode getNode() {
        if (gaddr == null) {
            return null;
        } else {
            return gaddr;
        }
    }

    @Override
    public String getStatus() {
        int per = (int) (100.0 * (double) current / (double) length);
        return per + "% @ " + current + "/" + length;
    }
}
