/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u_manage;

import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;
import pp4u.*;

/**
 * キー管理クラス - スレッドセーフ
 * @author tondol
 */
public class ManageKeys implements Runnable {

    static private ManageKeys instance = null;
    private ArrayList<InfoKey> keys;
    private volatile Thread thread = null;

    private ManageKeys() {
        keys = new ArrayList<InfoKey>();
    }

    /**
     * インスタンス化
     */
    static synchronized public ManageKeys getInstance() {
        if (instance == null) {
            instance = new ManageKeys();
        }
        return instance;
    }

    /**
     * キーリスト読み込み
     */
    public synchronized void load() {
        FileInputStream fst = null;
        ObjectInputStream ist = null;

        try {
            fst = new FileInputStream("pp4ukey.db");
            ist = new ObjectInputStream(fst);
            Object obj = null;
            while ((obj = ist.readObject()) != null) {
                add((InfoKey) obj);
            }
        } catch (EOFException eex) {
        } catch (Exception ex) {
            AppMain.debug("X キーリスト読込失敗");
            AppMain.debug("@ " + ex);
            // キーをすべて削除
            InfoKey[] arrayKeys = getArray();
            for (InfoKey key : arrayKeys) {
                remove(key);
            }
        } finally {
            try {
                if (ist != null) {
                    ist.close();
                }
            } catch (IOException iex) {
            }
        }
    }

    /**
     * キーリスト書き込み
     */
    public synchronized void dispose() {
        FileOutputStream fst = null;
        ObjectOutputStream ost = null;

        try {
            // ファイルを開く
            fst = new FileOutputStream("pp4ukey.db");
            ost = new ObjectOutputStream(fst);
            InfoKey[] arrayKeys = getArray();
            // キーリストを書き込む
            for (InfoKey key : arrayKeys) {
                if (key.isLocalKey()) {
                    ost.writeObject(key);
                }
            }
        } catch (Exception ex) {
            AppMain.debug("X キーリスト書込失敗");
            AppMain.debug("@ " + ex);
        } finally {
            try {
                if (ost != null) {
                    ost.flush();
                    ost.close();
                }
            } catch (IOException iex) {
            }
        }
    }

    /**
     * スキャン開始
     */
    public synchronized void start() {
        if (thread == null) {
            thread = new Thread(this);
            thread.start();
        }
    }

    /**
     * キー後処理
     */
    public synchronized void stop() {
        if (thread != null) {
            thread.interrupt();
        }
        // 仮想キーを削除
        InfoKey[] arrayKeys = getArray();
        for (InfoKey key : arrayKeys) {
            if (!key.isLocalKey()) {
                remove(key);
            }
        }
    }

    /**
     * ローカルキーを更新
     */
    public void run() {
        BoxMain.getInstance().setScan(true);
        BoxMain.getInstance().applyState();
        List<InfoKey> listKeys = null;

        try {
            // フォルダ内のファイルを取得
            File directory = ManageConfigure.getInstance().getUpload();
            File[] arrayFiles = directory.listFiles(new FilenameFilter() {
                // ファイルだけを取り出すフィルタ
                public boolean accept(File path, String name) {
                    return (new File(path + File.separator + name)).isFile();
                }
            });

            // 追加
            listKeys = searchLocal("");
            for (File file : arrayFiles) {
                boolean flag = true;
                for (InfoKey key : listKeys) {
                    if (key.getFilename().equals(file.getName())) {
                        flag = false;
                    }
                }
                if (flag) {
                    add(new InfoKey(file));
                }
            }
            // 削除
            listKeys = searchLocal("");
            for (InfoKey key : listKeys) {
                boolean flag = true;
                for (File file : arrayFiles) {
                    if (file.getName().equals(key.getFilename())) {
                        flag = false;
                    }
                }
                if (flag) {
                    remove(key);
                }
            }
            // 更新
            listKeys = searchLocal("");
            InfoNode localhost = AppMain.getLocalhost();
            for (InfoKey key : listKeys) {
                File file = new File(key.getUploadPath());
                if (key.getModified() != file.lastModified()) {
                    synchronized (keys) {
                        int index = keys.indexOf(key);
                        keys.set(index, new InfoKey(file));
                    }
                } else if (!key.getNode().equals(localhost)) {
                    key.setNode(localhost);
                }
            }

            // デバッグ表示
            AppMain.debug("O スキャン成功");
        } catch (Exception ex) {
            AppMain.debug("X スキャン失敗");
            AppMain.debug("@ " + ex);
            // キーをすべて削除
            InfoKey[] arrayKeys = getArray();
            for (InfoKey key : arrayKeys) {
                remove(key);
            }
        } finally {
            thread = null;
            BoxMain.getInstance().setScan(false);
            BoxMain.getInstance().applyState();
        }
    }

    /**
     * ローカルキーを検索
     */
    public List<InfoKey> searchLocal(String word) {
        InfoKey[] arrayKeys = getArray();
        ArrayList<InfoKey> list = new ArrayList<InfoKey>();
        for (InfoKey key : arrayKeys) {
            if (!key.isLocalKey()) {
                continue;
            }
            if (key.matches(word)) {
                list.add(key);
            }
        }
        return list;
    }

    /**
     * キー全体から検索
     */
    public List<InfoKey> searchAll(String word) {
        InfoKey[] arrayKeys = getArray();
        ArrayList<InfoKey> list = new ArrayList<InfoKey>();
        for (InfoKey key : arrayKeys) {
            if (key.matches(word)) {
                list.add(key);
            }
        }
        return list;
    }

    private InfoKey[] getArray() {
        synchronized (keys) {
            return keys.toArray(new InfoKey[0]);
        }
    }

    public int getSize() {
        synchronized (keys) {
            return keys.size();
        }
    }

    /**
     * キーを追加 - 重複を許可しない
     */
    public void add(InfoKey key) {
        synchronized (keys) {
            if (keys.contains(key)) {
                return;
            }
        }
        // 一番古いのを削除
        while (getSize() >= 1000) {
            InfoKey tmp = keys.get(0);
            remove(tmp);
        }
        // 追加
        synchronized (keys) {
            keys.add(key);
        }
        AppMain.debug("+ キー追加 <" + key.getFilename() + ">");
    }

    /**
     * キーを削除
     */
    public void remove(InfoKey key) {
        synchronized (keys) {
            keys.remove(key);
        }
        AppMain.debug("- キー削除 <" + key.getFilename() + ">");
    }
}
