/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u_manage;

import java.util.ArrayList;
import javax.swing.table.AbstractTableModel;
import pp4u.AppMain;
import pp4u.TaskAbstract;

/**
 * タスク管理クラス - スレッドセーフ
 * @author tondol
 */
public class ManageTasks extends AbstractTableModel implements Runnable {

    static ManageTasks instance = null;
    private ArrayList<TaskAbstract> tasks = null;
    private volatile Thread thread = null;

    private ManageTasks() {
        tasks = new ArrayList<TaskAbstract>();
    }

    /**
     * インスタンス化
     */
    static synchronized public ManageTasks getInstance() {
        if (instance == null) {
            instance = new ManageTasks();
        }
        return instance;
    }

    /**
     * タスク管理スレッド起動
     */
    public synchronized void load() {
        if (thread == null) {
            thread = new Thread(this);
            thread.start();
        //AppMain.debug("O タスク管理開始");
        }
    }

    /**
     * タスク管理スレッド終了
     */
    public synchronized void dispose() {
        if (thread != null) {
            thread.interrupt();
        }
        // タスクをすべて削除
        TaskAbstract[] arrayTasks = getArray();
        for (TaskAbstract task : arrayTasks) {
            remove(task);
        }
    }

    /**
     * タスク管理スレッド
     */
    public void run() {
        try {
            while (!thread.isInterrupted()) {
                callbacks();
                Thread.sleep(1 * 1000);
            }
        } catch (Exception ex) {
        } finally {
            thread = null;
        //AppMain.debug("O タスク管理終了");
        }
    }

    /**
     * コールバックメソッド
     */
    private void callbacks() {
        // 終了したタスクを削除する
        TaskAbstract[] arrayTasks = getArray();
        for (TaskAbstract task : arrayTasks) {
            if (!task.isAlive()) {
                remove(task);
            }
        }
        int lastIndex = arrayTasks.length - 1;
        fireTableRowsUpdated(0, lastIndex);
    }

    /**
     * インデックスからタスクを取得
     */
    public TaskAbstract getTask(int index) {
        synchronized (tasks) {
            return tasks.get(index);
        }
    }

    /**
     * タスクリストを配列として取得
     */
    public TaskAbstract[] getArray() {
        synchronized (tasks) {
            return tasks.toArray(new TaskAbstract[0]);
        }
    }

    /**
     * タスク追加 - 重複を許さない
     */
    public void add(TaskAbstract task) {
        int addIndex;
        synchronized (tasks) {
            if (tasks.contains(task)) {
                task.stop();
                return;
            }
            addIndex = tasks.size();
            tasks.add(task);
        }
        fireTableRowsInserted(addIndex, addIndex);
        AppMain.debug("O タスク追加 <" + task.getFilename() + ">");
    }

    /**
     * タスク削除
     */
    public void remove(TaskAbstract task) {
        task.stop();
        int removeIndex;
        synchronized (tasks) {
            removeIndex = tasks.indexOf(task);
            tasks.remove(task);
        }
        fireTableRowsDeleted(removeIndex, removeIndex);
        AppMain.debug("X タスク削除 <" + task.getFilename() + ">");
    }

    //////////////////////////////////////////////
    // TableModelの実装もね
    //////////////////////////////////////////////
    @Override
    public int getColumnCount() {
        return 4;
    }

    @Override
    public String getColumnName(int column) {
        String ret = null;
        switch (column) {
            case 0:
                ret = "種別";
                break;
            case 1:
                ret = "ファイル名";
                break;
            case 2:
                ret = "接続先";
                break;
            case 3:
                ret = "ステータス";
                break;
        }
        return ret;
    }

    public int getRowCount() {
        synchronized (tasks) {
            return tasks.size();
        }
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        Object obj = null;
        TaskAbstract task = null;

        // 要素取得
        synchronized (tasks) {
            if (rowIndex >= tasks.size()) {
                return null;
            }
            task = getTask(rowIndex);
        }

        switch (columnIndex) {
            case 0:
                obj = task.getType();
                break;
            case 1:
                obj = task.getFilename();
                break;
            case 2:
                obj = task.getNode();
                break;
            case 3:
                obj = task.getStatus();
                break;
        }
        return obj;
    }
}
